/* $Id: internal.h,v 1.1 2000/03/13 04:51:41 ajapted Exp $
******************************************************************************

   LibGFFD - internal definitions

   Copyright (C) 2000  Andrew Apted    [andrew.apted@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _GFFD_INTERNAL_INTERNAL_H
#define _GFFD_INTERNAL_INTERNAL_H


#define _INTERNAL_LIBGFFD  1

typedef struct gffd_context *gffd_context_t;


#include "config.h"
#include <gffd/gffd.h>

#include "list.h"
#include "debug.h"


/* Macros */

#ifndef MIN
#define MIN(a,b)  ((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b)  ((a) > (b) ? (a) : (b))
#endif
#ifndef ABS
#define ABS(x)  (((x) < 0) ? -(x) : (x))
#endif
#ifndef LIMIT
#define LIMIT(lo,x,hi)  (((x) <= (lo)) ? (lo) : (((x) >= (hi)) ? (hi) : (x)))
#endif
#ifndef SWAP2
#define SWAP2(type,a,b)  do { type t; t=(a); (a)=(b); (b)=t; } while(0) 
#endif


/* Constants */

/* ... none yet */


/* Rule structures */

typedef struct gffd_name
{
  d_node_t n;

  /* score, from 1 to 3 */
  int score;

  const char *name;
} 
gffd_name_t;

typedef struct gffd_part
{
  d_node_t n;

  int offset;
  int length;

  /* interleaved mask/match values */
  unsigned char *data;
} 
gffd_part_t;

typedef struct gffd_magic
{
  d_node_t n;
  
  /* score, from 1 to 3 */
  int score;

  d_list_t /* of gffd_part_t */ parts;
} 
gffd_magic_t;

typedef struct gffd_rule
{
  d_node_t n;

  const char *class;
  const char *type;
  const char *MIME;

  d_list_t /* of gffd_name_t  */ names; 
  d_list_t /* of gffd_magic_t */ magics;
} 
gffd_rule_t;


/* Context structure */

typedef struct gffd_match
{
  short name_score;
  short magic_score;

  gffd_rule_t *rule;
}
gffd_match_t;

typedef struct gffd_context
{
  /* gffdBegin has been called */
  int begun;

  /* no matches have been performed yet ? */
  int full_set;

  /* have we matched names or magic yet ? */
  int done_names;
  int done_magic;

  /* current count of matches */
  int count;

  /* current set of matches */
  gffd_match_t * matches;
}
gffd_context;


/* Internal API calls
 */

__BEGIN_DECLS

void _gffdInitRules(void);
int  _gffdLoadRules(const char *filename);
void _gffdFreeRules(void);
d_list_t /* of gffd_rule_t */ * _gffdGetRules(void);

__END_DECLS


#endif  /* _GFFD_INTERNAL_INTERNAL_H */
